@@ -4,8 +4,8 @@
 from __future__ import print_function
 from __future__ import unicode_literals
 
+import array
 import calendar
-import ftplib
 import io
 import itertools
 import socket
@@ -14,12 +14,18 @@ import typing
 from collections import OrderedDict
 from contextlib import contextmanager
 from ftplib import FTP
+
+try:
+    from ftplib import FTP_TLS
+except ImportError as err:
+    FTP_TLS = err  # type: ignore
 from ftplib import error_perm
 from ftplib import error_temp
 from typing import cast
 
 from six import PY2
 from six import text_type
+from six import raise_from
 
 from . import errors
 from .base import FS
@@ -36,7 +42,8 @@ from .path import normpath
 from .path import split
 from . import _ftp_parse as ftp_parse
 
-if False:  # typing.TYPE_CHECKING
+if typing.TYPE_CHECKING:
+    import mmap
     import ftplib
     from typing import (
         Any,
@@ -45,7 +52,6 @@ if False:  # typing.TYPE_CHECKING
         ContextManager,
         Iterable,
         Iterator,
-        Collection,
         Container,
         Dict,
         List,
@@ -77,6 +83,8 @@ def ftp_errors(fs, path=None):
         raise errors.RemoteConnectionError(
             msg="unable to connect to {}".format(fs.host)
         )
+    except EOFError:
+        raise errors.RemoteConnectionError(msg="lost connection to {}".format(fs.host))
     except error_temp as error:
         if path is not None:
             raise errors.ResourceError(
@@ -101,7 +109,7 @@ def manage_ftp(ftp):
     finally:
         try:
             ftp.quit()
-        except:  # pragma: no cover
+        except Exception:  # pragma: no cover
             pass
 
 
@@ -235,8 +243,18 @@ class FTPFile(io.RawIOBase):
                 remaining -= len(chunk)
         return b"".join(chunks)
 
-    def readline(self, size=-1):
-        # type: (int) -> bytes
+    def readinto(self, buffer):
+        # type: (Union[bytearray, memoryview, array.array[Any], mmap.mmap]) -> int
+        data = self.read(len(buffer))
+        bytes_read = len(data)
+        if isinstance(buffer, array.array):
+            buffer[:bytes_read] = array.array(buffer.typecode, data)
+        else:
+            buffer[:bytes_read] = data  # type: ignore
+        return bytes_read
+
+    def readline(self, size=None):
+        # type: (Optional[int]) -> bytes
         return next(line_iterator(self, size))  # type: ignore
 
     def readlines(self, hint=-1):
@@ -255,10 +273,13 @@ class FTPFile(io.RawIOBase):
         return self.mode.writing
 
     def write(self, data):
-        # type: (bytes) -> int
+        # type: (Union[bytes, memoryview, array.array[Any], mmap.mmap]) -> int
         if not self.mode.writing:
             raise IOError("File not open for writing")
 
+        if isinstance(data, array.array):
+            data = data.tobytes()
+
         with self._lock:
             conn = self.write_conn
             data_pos = 0
@@ -274,8 +295,16 @@ class FTPFile(io.RawIOBase):
         return data_pos
 
     def writelines(self, lines):
-        # type: (Iterable[bytes]) -> None
-        self.write(b"".join(lines))
+        # type: (Iterable[Union[bytes, memoryview, array.array[Any], mmap.mmap]]) -> None  # noqa: E501
+        if not self.mode.writing:
+            raise IOError("File not open for writing")
+        data = bytearray()
+        for line in lines:
+            if isinstance(line, array.array):
+                data.extend(line.tobytes())
+            else:
+                data.extend(line)  # type: ignore
+        self.write(data)
 
     def truncate(self, size=None):
         # type: (Optional[int]) -> int
@@ -325,16 +354,35 @@ class FTPFile(io.RawIOBase):
 class FTPFS(FS):
     """A FTP (File Transport Protocol) Filesystem.
 
-    Arguments:
-        host (str): A FTP host, e.g. ``'ftp.mirror.nl'``.
-        user (str): A username (default is ``'anonymous'``).
-        passwd (str): Password for the server, or `None` for anon.
-        acct (str): FTP account.
-        timeout (int): Timeout for contacting server (in seconds,
-            defaults to 10).
-        port (int): FTP port number (default 21).
-        proxy (str, optional): An FTP proxy, or ``None`` (default)
-            for no proxy.
+    Optionally, the connection can be made securely via TLS. This is known as
+    FTPS, or FTP Secure. TLS will be enabled when using the ftps:// protocol,
+    or when setting the `tls` argument to True in the constructor.
+
+    Examples:
+        Create with the constructor::
+
+            >>> from fs.ftpfs import FTPFS
+            >>> ftp_fs = FTPFS("demo.wftpserver.com")
+
+        Or via an FS URL::
+
+            >>> ftp_fs = fs.open_fs('ftp://test.rebex.net')
+
+        Or via an FS URL, using TLS::
+
+            >>> ftp_fs = fs.open_fs('ftps://demo.wftpserver.com')
+
+        You can also use a non-anonymous username, and optionally a
+        password, even within a FS URL::
+
+            >>> ftp_fs = FTPFS("test.rebex.net", user="demo", passwd="password")
+            >>> ftp_fs = fs.open_fs('ftp://demo:password@test.rebex.net')
+
+        Connecting via a proxy is supported. If using a FS URL, the proxy
+        URL will need to be added as a URL parameter::
+
+            >>> ftp_fs = FTPFS("ftp.ebi.ac.uk", proxy="test.rebex.net")
+            >>> ftp_fs = fs.open_fs('ftp://ftp.ebi.ac.uk/?proxy=test.rebex.net')
 
     """
 
@@ -356,8 +404,24 @@ class FTPFS(FS):
         timeout=10,  # type: int
         port=21,  # type: int
         proxy=None,  # type: Optional[Text]
+        tls=False,  # type: bool
     ):
         # type: (...) -> None
+        """Create a new `FTPFS` instance.
+
+        Arguments:
+            host (str): A FTP host, e.g. ``'ftp.mirror.nl'``.
+            user (str): A username (default is ``'anonymous'``).
+            passwd (str): Password for the server, or `None` for anon.
+            acct (str): FTP account.
+            timeout (int): Timeout for contacting server (in seconds,
+                defaults to 10).
+            port (int): FTP port number (default 21).
+            proxy (str, optional): An FTP proxy, or ``None`` (default)
+                for no proxy.
+            tls (bool): Attempt to use FTP over TLS (FTPS) (default: False)
+
+        """
         super(FTPFS, self).__init__()
         self._host = host
         self._user = user
@@ -366,6 +430,10 @@ class FTPFS(FS):
         self.timeout = timeout
         self.port = port
         self.proxy = proxy
+        self.tls = tls
+
+        if self.tls and isinstance(FTP_TLS, Exception):
+            raise_from(errors.CreateFailed("FTP over TLS not supported"), FTP_TLS)
 
         self.encoding = "latin-1"
         self._ftp = None  # type: Optional[FTP]
@@ -396,8 +464,7 @@ class FTPFS(FS):
     @classmethod
     def _parse_features(cls, feat_response):
         # type: (Text) -> Dict[Text, Text]
-        """Parse a dict of features from FTP feat response.
-        """
+        """Parse a dict of features from FTP feat response."""
         features = {}
         if feat_response.split("-")[0] == "211":
             for line in feat_response.splitlines():
@@ -408,13 +475,16 @@ class FTPFS(FS):
 
     def _open_ftp(self):
         # type: () -> FTP
-        """Open a new ftp object.
-        """
-        _ftp = FTP()
+        """Open a new ftp object."""
+        _ftp = FTP_TLS() if self.tls else FTP()
         _ftp.set_debuglevel(0)
         with ftp_errors(self):
             _ftp.connect(self.host, self.port, self.timeout)
             _ftp.login(self.user, self.passwd, self.acct)
+            try:
+                _ftp.prot_p()  # type: ignore
+            except AttributeError:
+                pass
             self._features = {}
             try:
                 feat_response = _decode(_ftp.sendcmd("FEAT"), "latin-1")
@@ -440,18 +510,24 @@ class FTPFS(FS):
     def ftp_url(self):
         # type: () -> Text
         """Get the FTP url this filesystem will open."""
-        url = (
-            "ftp://{}".format(self.host)
-            if self.port == 21
-            else "ftp://{}:{}".format(self.host, self.port)
-        )
+        if self.port == 21:
+            _host_part = self.host
+        else:
+            _host_part = "{}:{}".format(self.host, self.port)
+
+        if self.user == "anonymous" or self.user is None:
+            _user_part = ""
+        else:
+            _user_part = "{}:{}@".format(self.user, self.passwd)
+
+        scheme = "ftps" if self.tls else "ftp"
+        url = "{}://{}{}".format(scheme, _user_part, _host_part)
         return url
 
     @property
     def ftp(self):
         # type: () -> FTP
-        """~ftplib.FTP: the underlying FTP client.
-        """
+        """~ftplib.FTP: the underlying FTP client."""
         return self._get_ftp()
 
     def geturl(self, path, purpose="download"):
@@ -469,10 +545,9 @@ class FTPFS(FS):
         return self._ftp
 
     @property
-    def features(self):
+    def features(self):  # noqa: D401
         # type: () -> Dict[Text, Text]
-        """dict: features of the remote FTP server.
-        """
+        """`dict`: Features of the remote FTP server."""
         self._get_ftp()
         return self._features
 
@@ -494,8 +569,7 @@ class FTPFS(FS):
     @property
     def supports_mlst(self):
         # type: () -> bool
-        """bool: whether the server supports MLST feature.
-        """
+        """bool: whether the server supports MLST feature."""
         return "MLST" in self.features
 
     def create(self, path, wipe=False):
@@ -513,8 +587,7 @@ class FTPFS(FS):
     @classmethod
     def _parse_ftp_time(cls, time_text):
         # type: (Text) -> Optional[int]
-        """Parse a time from an ftp directory listing.
-        """
+        """Parse a time from an ftp directory listing."""
         try:
             tm_year = int(time_text[0:4])
             tm_month = int(time_text[4:6])
@@ -575,7 +648,7 @@ class FTPFS(FS):
                 details["created"] = cls._parse_ftp_time(facts["create"])
             yield raw_info
 
-    if False:  # typing.TYPE_CHECKING
+    if typing.TYPE_CHECKING:
 
         def opendir(self, path, factory=None):
             # type: (_F, Text, Optional[_OpendirFactory]) -> SubFS[_F]
@@ -677,7 +750,7 @@ class FTPFS(FS):
                     raise errors.FileExpected(path)
                 if _mode.exclusive:
                     raise errors.FileExists(path)
-            ftp_file = FTPFile(self, _path, mode)
+            ftp_file = FTPFile(self, _path, _mode.to_platform_bin())
         return ftp_file  # type: ignore
 
     def remove(self, path):
@@ -728,9 +801,8 @@ class FTPFS(FS):
                     for raw_info in self._parse_mlsx(lines):
                         yield Info(raw_info)
                     return
-            with self._lock:
-                for info in self._read_dir(_path).values():
-                    yield info
+            for info in self._read_dir(_path).values():
+                yield info
 
     def scandir(
         self,
@@ -751,11 +823,10 @@ class FTPFS(FS):
         # type: (Text, BinaryIO, Optional[int], **Any) -> None
         _path = self.validatepath(path)
         with self._lock:
-            with self._manage_ftp() as ftp:
-                with ftp_errors(self, path):
-                    ftp.storbinary(
-                        str("STOR ") + _encode(_path, self.ftp.encoding), file
-                    )
+            with ftp_errors(self, path):
+                self.ftp.storbinary(
+                    str("STOR ") + _encode(_path, self.ftp.encoding), file
+                )
 
     def writebytes(self, path, contents):
         # type: (Text, ByteString) -> None
