@@ -12,16 +12,15 @@ from .base import Opener
 from .registry import registry
 from .errors import NotWriteable
 
-if False:  # typing.TYPE_CHECKING
+if typing.TYPE_CHECKING:
     from typing import Text
     from .parse import ParseResult
-    from ..zipfs import ZipFS
+    from ..zipfs import ZipFS  # noqa: F401
 
 
 @registry.install
 class ZipOpener(Opener):
-    """`ZipFS` opener.
-    """
+    """`ZipFS` opener."""
 
     protocols = ["zip"]
 
