@@ -16,7 +16,7 @@ import typing
 
 from .errors import IllegalBackReference
 
-if False:  # typing.TYPE_CHECKING
+if typing.TYPE_CHECKING:
     from typing import List, Text, Tuple
 
 
@@ -64,11 +64,11 @@ def normpath(path):
         >>> normpath("/foo//bar/frob/../baz")
         '/foo/bar/baz'
         >>> normpath("foo/../../bar")
-        Traceback (most recent call last)
+        Traceback (most recent call last):
             ...
-        IllegalBackReference: path 'foo/../../bar' contains back-references outside of filesystem"
+        fs.errors.IllegalBackReference: path 'foo/../../bar' contains back-references outside of filesystem
 
-    """
+    """  # noqa: E501
     if path in "/":
         return path
 
@@ -86,6 +86,7 @@ def normpath(path):
             else:
                 components.append(component)
     except IndexError:
+        # FIXME (@althonos): should be raised from the IndexError
         raise IllegalBackReference(path)
     return prefix + "/".join(components)
 
@@ -509,7 +510,7 @@ def forcedir(path):
         >>> forcedir("foo/bar/")
         'foo/bar/'
         >>> forcedir("foo/spam.txt")
-        'foo/spam.txt'
+        'foo/spam.txt/'
 
     """
     if not path.endswith("/"):
