@@ -79,8 +79,7 @@ def render(
 
     def write(line):
         # type: (Text) -> None
-        """Write a line to the output.
-        """
+        """Write a line to the output."""
         print(line, file=file)
 
     # FIXME(@althonos): define functions using `with_color` and
@@ -88,32 +87,28 @@ def render(
 
     def format_prefix(prefix):
         # type: (Text) -> Text
-        """Format the prefix lines.
-        """
+        """Format the prefix lines."""
         if not with_color:
             return prefix
         return "\x1b[32m%s\x1b[0m" % prefix
 
     def format_dirname(dirname):
         # type: (Text) -> Text
-        """Format a directory name.
-        """
+        """Format a directory name."""
         if not with_color:
             return dirname
         return "\x1b[1;34m%s\x1b[0m" % dirname
 
     def format_error(msg):
         # type: (Text) -> Text
-        """Format an error.
-        """
+        """Format an error."""
         if not with_color:
             return msg
         return "\x1b[31m%s\x1b[0m" % msg
 
     def format_filename(fname):
         # type: (Text) -> Text
-        """Format a filename.
-        """
+        """Format a filename."""
         if not with_color:
             return fname
         if fname.startswith("."):
@@ -122,26 +117,23 @@ def render(
 
     def sort_key_dirs_first(info):
         # type: (Info) -> Tuple[bool, Text]
-        """Get the info sort function with directories first.
-        """
+        """Get the info sort function with directories first."""
         return (not info.is_dir, info.name.lower())
 
     def sort_key(info):
         # type: (Info) -> Text
-        """Get the default info sort function using resource name.
-        """
+        """Get the default info sort function using resource name."""
         return info.name.lower()
 
     counts = {"dirs": 0, "files": 0}
 
     def format_directory(path, levels):
         # type: (Text, List[bool]) -> None
-        """Recursive directory function.
-        """
+        """Recursive directory function."""
         try:
             directory = sorted(
                 fs.filterdir(path, exclude_dirs=exclude, files=filter),
-                key=sort_key_dirs_first if dirs_first else sort_key,
+                key=sort_key_dirs_first if dirs_first else sort_key,  # type: ignore
             )
         except Exception as error:
             prefix = (
