@@ -44,7 +44,7 @@ if typing.TYPE_CHECKING:
 
 
 class _ZipExtFile(RawWrapper):
-    def __init__(self, fs, name):
+    def __init__(self, fs, name):  # noqa: D107
         # type: (ReadZipFS, Text) -> None
         self._zip = _zip = fs._zip
         self._end = _zip.getinfo(name).file_size
@@ -124,11 +124,11 @@ class _ZipExtFile(RawWrapper):
 class ZipFS(WrapFS):
     """Read and write zip files.
 
-    There are two ways to open a ZipFS for the use cases of reading
+    There are two ways to open a `ZipFS` for the use cases of reading
     a zip file, and creating a new one.
 
-    If you open the ZipFS with  ``write`` set to `False` (the default)
-    then the filesystem will be a read only filesystem which maps to
+    If you open the `ZipFS` with  ``write`` set to `False` (the default)
+    then the filesystem will be a read-only filesystem which maps to
     the files and directories within the zip file. Files are
     decompressed on the fly when you open them.
 
@@ -137,12 +137,12 @@ class ZipFS(WrapFS):
         with ZipFS('foo.zip') as zip_fs:
             readme = zip_fs.readtext('readme.txt')
 
-    If you open the ZipFS with ``write`` set to `True`, then the ZipFS
-    will be a empty temporary filesystem. Any files / directories you
-    create in the ZipFS will be written in to a zip file when the ZipFS
+    If you open the `ZipFS` with ``write`` set to `True`, then the `ZipFS`
+    will be an empty temporary filesystem. Any files / directories you
+    create in the `ZipFS` will be written in to a zip file when the `ZipFS`
     is closed.
 
-    Here's how you might write a new zip file containing a readme.txt
+    Here's how you might write a new zip file containing a ``readme.txt``
     file::
 
         with ZipFS('foo.zip', write=True) as new_zip:
@@ -158,18 +158,20 @@ class ZipFS(WrapFS):
             (default) to read an existing zip file.
         compression (int): Compression to use (one of the constants
             defined in the `zipfile` module in the stdlib).
-        temp_fs (str): An FS URL for the temporary filesystem used to
-            store data prior to zipping.
+        temp_fs (str or FS): An FS URL or an FS instance to use to
+            store data prior to zipping. Defaults to creating a new
+            `~fs.tempfs.TempFS`.
 
     """
 
-    def __new__(
+    # TODO: __new__ returning different types may be too 'magical'
+    def __new__(  # type: ignore
         cls,
         file,  # type: Union[Text, BinaryIO]
         write=False,  # type: bool
         compression=zipfile.ZIP_DEFLATED,  # type: int
         encoding="utf-8",  # type: Text
-        temp_fs="temp://__ziptemp__",  # type: Text
+        temp_fs="temp://__ziptemp__",  # type: Union[Text, FS]
     ):
         # type: (...) -> FS
         # This magic returns a different class instance based on the
@@ -190,23 +192,22 @@ class ZipFS(WrapFS):
             compression=zipfile.ZIP_DEFLATED,  # type: int
             encoding="utf-8",  # type: Text
             temp_fs="temp://__ziptemp__",  # type: Text
-        ):
+        ):  # noqa: D107
             # type: (...) -> None
             pass
 
 
 @six.python_2_unicode_compatible
 class WriteZipFS(WrapFS):
-    """A writable zip file.
-    """
+    """A writable zip file."""
 
     def __init__(
         self,
         file,  # type: Union[Text, BinaryIO]
         compression=zipfile.ZIP_DEFLATED,  # type: int
         encoding="utf-8",  # type: Text
-        temp_fs="temp://__ziptemp__",  # type: Text
-    ):
+        temp_fs="temp://__ziptemp__",  # type: Union[Text, FS]
+    ):  # noqa: D107
         # type: (...) -> None
         self._file = file
         self.compression = compression
@@ -275,8 +276,7 @@ class WriteZipFS(WrapFS):
 
 @six.python_2_unicode_compatible
 class ReadZipFS(FS):
-    """A readable zip file.
-    """
+    """A readable zip file."""
 
     _meta = {
         "case_insensitive": True,
@@ -289,7 +289,7 @@ class ReadZipFS(FS):
     }
 
     @errors.CreateFailed.catch_all
-    def __init__(self, file, encoding="utf-8"):
+    def __init__(self, file, encoding="utf-8"):  # noqa: D107
         # type: (Union[BinaryIO, Text], Text) -> None
         super(ReadZipFS, self).__init__()
         self._file = file
@@ -307,8 +307,7 @@ class ReadZipFS(FS):
 
     def _path_to_zip_name(self, path):
         # type: (Text) -> str
-        """Convert a path to a zip file name.
-        """
+        """Convert a path to a zip file name."""
         path = relpath(normpath(path))
         if self._directory.isdir(path):
             path = forcedir(path)
@@ -319,8 +318,7 @@ class ReadZipFS(FS):
     @property
     def _directory(self):
         # type: () -> MemoryFS
-        """`MemoryFS`: a filesystem with the same folder hierarchy as the zip.
-        """
+        """`MemoryFS`: a filesystem with the same folder hierarchy as the zip."""
         self.check()
         with self._lock:
             if self._directory_fs is None:
